#property copyright "ArteInVolo"
#property version "1.00"

#property indicator_separate_window 
#property indicator_buffers 1
#property indicator_plots 1

#property indicator_type1 DRAW_LINE
#property indicator_color1 clrBlue
#property indicator_style1 STYLE_SOLID
#property indicator_width1 1
#property indicator_label1 "Index"

#include "PCA.mqh"

// *** Input *** //

input int InpDepth = 1000;
input int InpOosDepth = 0;
input bool InpUseMaxEigen = true;
input bool InpUseCorrelation = true;
input bool InpShowSynthetics = true;
input ENUM_TIMEFRAMES InpPeriod = PERIOD_CURRENT;
input string InpUnit = "USD";
input string InpSymbols = "AUDUSD,EURUSD,GBPUSD,USDCHF,USDJPY,USDCAD,NZDUSD,USDSEK,USDSGD,USDMXN,USDNOK";

// *** Globals *** //

double iChart[];
int iOrder, iDepth, iSourceDepth;
SSeries iSeries[], iBase[], iCovs[], iVectors[], iEigens[], iCors[], iSynthetics[], iInversions[], iEstimators[];

// *** Implementation *** //

int OnInit()
{
    iSourceDepth = InpDepth;
    iDepth = iSourceDepth + InpOosDepth;
    iOrder = ListSymbols(iSeries, GetSymbolNames(InpSymbols, InpUnit));

    SetIndexBuffer(0, iChart, INDICATOR_DATA);
    ArraySetAsSeries(iChart, true);

    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
    IndicatorSetString(INDICATOR_SHORTNAME, "PCA");

    SetupMatrix(iCovs, iOrder, iDepth);
    SetupMatrix(iEigens, iOrder, iDepth);
    SetupMatrix(iVectors, iOrder, iDepth);
    SetupMatrix(iCors, iOrder, iDepth);
    SetupMatrix(iBase, iOrder, iDepth);
    SetupMatrix(iSeries, iOrder, iDepth);
    SetupMatrix(iSynthetics, iOrder, iDepth);
    SetupMatrix(iInversions, iOrder, iDepth);
    SetupMatrix(iEstimators, iOrder, iDepth);
    
    ClearMatrix(iCovs, iOrder, iDepth);
    ClearMatrix(iEigens, iOrder, iDepth);
    ClearMatrix(iVectors, iOrder, iDepth);
    ClearMatrix(iCors, iOrder, iDepth);
    ClearMatrix(iBase, iOrder, iDepth);
    ClearMatrix(iSeries, iOrder, iDepth);
    ClearMatrix(iSynthetics, iOrder, iDepth);
    ClearMatrix(iInversions, iOrder, iDepth);
    ClearMatrix(iEstimators, iOrder, iDepth);

    SetupPCA(iOrder, iOrder);
    
    EventSetTimer(1);

    return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
    HidePanel();
}

void OnTimer()
{
    Display();
}

int OnCalculate(
    const int bars,
    const int counted,
    const datetime &time[],
    const double &open[],
    const double& high[],
    const double& low[],
    const double &close[],
    const long &tickVolume[],
    const long &volume[],
    const int &spread[])
{
    return(bars);
}

void Display()
{
    int index = 0;
    int vectorIndex = 0;
    int chartBars = ArraySize(iChart);

    if (Synchronize(iSeries, InpPeriod, iOrder, iDepth) > 0)
    {
        datetime oosTime = GetLastTime(iSeries, InpPeriod, iOrder) - PeriodSeconds(InpPeriod) * InpOosDepth;

        SetLine("OOS", oosTime);
        GetBaseMatrix(iSeries, iBase, iOrder, iDepth);
        GetCovMatrix(iBase, iCovs, iOrder, iSourceDepth);
        GetInverseMatrix(iCovs, iInversions, iOrder);
        //GetEstimator(iSeries, iCovs, iInversions, iOrder, iDepth);
        GetEigenValues(iInversions, iEigens, iVectors, iOrder);
        vectorIndex = GetOptimalVector(iEigens, iOrder, InpUseMaxEigen);
        GetIndexValues(iSeries, iVectors, iSynthetics, iOrder, iDepth, vectorIndex);

        if (InpUseCorrelation)
        {
            GetCorMatrix(iInversions, iCors, iOrder);
            GetEigenValues(iCors, iEigens, iVectors, iOrder);
            vectorIndex = GetOptimalVector(iEigens, iOrder, InpUseMaxEigen);
        }

        ShowVector(iSeries, iVectors, iOrder, vectorIndex);
    }
    
    for (int k = iDepth - 1; k > 0 && chartBars-- > 0; k--)
    {
        iChart[index] = iSynthetics[0].mPoints[k];
        index++;
    }

    ChartRedraw();
}